@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) 2008-2013, Luis Pedro Coelho <luis@luispedro.org>
+# Copyright (C) 2008-2015, Luis Pedro Coelho <luis@luispedro.org>
 # vim: set ts=4 sts=4 sw=4 expandtab smartindent:
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -26,12 +26,11 @@ file_store : file-system based data store & locks.
 
 
 import os
-from os import path, mkdir, fdopen
+from os import path, mkdir
 from os.path import dirname, exists
 import errno
 import tempfile
 import shutil
-import six
 
 from .base import base_store
 from jug.backends.encode import encode_to, decode_from
@@ -53,6 +52,19 @@ def create_directories(dname):
             raise
 
 
+def _fsync_dir(fname):
+    import errno
+    parent = dirname(fname)
+    fd = os.open(parent, os.O_RDONLY)
+    try:
+        os.fsync(fd)
+    except OSError as err:
+        if err.errno != errno.EINVAL:
+            raise
+    finally:
+        os.close(fd)
+
+
 class file_store(base_store):
     def __init__(self, dname):
         '''
@@ -63,6 +75,10 @@ class file_store(base_store):
         if dname.endswith('/'): dname = dname[:-1]
         self.jugdir = dname
 
+    def __repr__(self):
+        return 'file_store({})'.format(self.jugdir)
+    __str__ = __repr__
+
     def create(self):
         '''
         Recursively create directories.
@@ -91,15 +107,16 @@ class file_store(base_store):
         '''
         store.dump(object, name)
 
-        Performs the same as
+        Performs roughly the same as
 
         pickle.dump(object, open(name,'w'))
 
-        but does it in a way that is guaranteed to be atomic even over NFS.
+        but does it in a way that is guaranteed to be atomic even over NFS and
+        using compression on the disk for faster access.
         '''
+        self._maybe_create()
         name = self._getfname(name)
         create_directories(dirname(name))
-        self._maybe_create()
         fd, fname = tempfile.mkstemp('.jugtmp', 'jugtemp', self.tempdir())
         output = os.fdopen(fd, 'wb')
         try:
@@ -109,6 +126,7 @@ class file_store(base_store):
                 output.flush()
                 os.fsync(output.fileno())
                 output.close()
+                _fsync_dir(fname)
                 os.rename(fname, name)
                 return
         except ImportError:
@@ -124,6 +142,7 @@ class file_store(base_store):
         output.close()
 
         # Rename is atomic even over NFS.
+        _fsync_dir(fname)
         os.rename(fname, name)
 
     def list(self):
@@ -339,6 +358,9 @@ class file_based_lock(object):
     '''
 
     def __init__(self, jugdir, name):
+        import six
+        if type(name) != six.text_type:
+            name = six.text_type(name, 'utf-8')
         self.fullname = path.join(jugdir, 'locks', '{0}.lock'.format(name))
 
     def get(self):
