@@ -1,6 +1,6 @@
 #!/usr/bin/python
 # -*- coding: utf-8 -*-
-# Copyright (C) 2008-2013, Luis Pedro Coelho <luis@luispedro.org>
+# Copyright (C) 2008-2014, Luis Pedro Coelho <luis@luispedro.org>
 # vim: set ts=4 sts=4 sw=4 expandtab smartindent:
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -31,6 +31,7 @@ import logging
 
 from . import task
 from .task import Task
+from .hooks import jug_hook, register_hook
 from .io import print_task_summary_table
 from .subcommands.status import status
 from .subcommands.webstatus import webstatus
@@ -111,7 +112,7 @@ def invalidate(store, options):
 def _sigterm(_,__):
     sys.exit(1)
 
-def execution_loop(tasks, options, tasks_executed, tasks_loaded):
+def execution_loop(tasks, options):
     from time import sleep
 
     logging.info('Execute start (%s tasks)' % len(tasks))
@@ -147,22 +148,24 @@ def execution_loop(tasks, options, tasks_executed, tasks_loaded):
         for t in upnext:
             if t.can_load():
                 logging.info('Loadable %s...' % t.name)
-                tasks_loaded[t.name] += 1
+                jug_hook('execute.task-loadable', (t,))
                 continue
             locked = False
             try:
                 locked = t.lock()
                 if t.can_load(): # This can be true if the task ran between the check above and this one
                     logging.info('Loadable %s...' % t.name)
-                    tasks_loaded[t.name] += 1
+                    jug_hook('execute.task-loadable', (t,))
                 elif locked:
                     logging.info('Executing %s...' % t.name)
+                    jug_hook('execute.task-pre-execute', (t,))
+
                     t.run(debug_mode=options.debug)
-                    tasks_executed[t.name] += 1
-                    if options.aggressive_unload:
-                        t.unload_recursive()
+                    jug_hook('execute.task-executed1', (t,))
                 else:
                     logging.info('Already in execution %s...' % t.name)
+            except SystemExit:
+                raise
             except Exception as e:
                 if options.pdb:
                     import sys
@@ -191,7 +194,7 @@ def execution_loop(tasks, options, tasks_executed, tasks_loaded):
                             import IPython.Debugger
                             shell = IPython.Shell.IPShell(argv=[''])
                             ip = IPython.ipapi.get()
-                            debugger=IPythong.Debugger.Pdb(ip.options.colors)
+                            debugger = IPython.Debugger.Pdb(ip.options.colors)
                     except ImportError:
                         #Fallback to standard debugger
                         import pdb
@@ -210,6 +213,19 @@ def execution_loop(tasks, options, tasks_executed, tasks_loaded):
                     raise
             finally:
                 if locked: t.unlock()
+
+class TaskStats(object):
+    def __init__(self):
+        self.loaded = defaultdict(int)
+        self.executed = defaultdict(int)
+        register_hook('execute.task-loadable', self.loadable)
+        register_hook('execute.task-executed1', self.executed1)
+
+    def loadable(self, t):
+        self.loaded[t.name] += 1
+    def executed1(self, t):
+        self.executed[t.name] += 1
+
 def execute(options):
     '''
     execute(options)
@@ -220,10 +236,12 @@ def execute(options):
 
     signal(SIGTERM,_sigterm)
     tasks = task.alltasks
-    tasks_executed = defaultdict(int)
-    tasks_loaded = defaultdict(int)
+    tstats = TaskStats()
     store = None
     noprogress = 0
+    if options.aggressive_unload:
+        register_hook('execute.task-executed1', lambda t : t.unload_recursive())
+
     while noprogress < 2:
         del tasks[:]
         store,jugspace = init(options.jugfile, options.jugdir, store=store)
@@ -232,9 +250,9 @@ def execute(options):
                 # Trigger hash computation:
                 t.hash()
 
-        previous = sum(tasks_executed.values())
-        execution_loop(tasks, options, tasks_executed, tasks_loaded)
-        after = sum(tasks_executed.values())
+        previous = sum(tstats.executed.values())
+        execution_loop(tasks, options)
+        after = sum(tstats.executed.values())
         done = not jugspace.get('__jug__hasbarrier__', False)
         if done:
             break
@@ -247,7 +265,7 @@ def execute(options):
 
 
 
-    print_task_summary_table(options, [("Executed", tasks_executed), ("Loaded", tasks_loaded)])
+    print_task_summary_table(options, [("Executed", tstats.executed), ("Loaded", tstats.loaded)])
 
 def cleanup(store, options):
     '''
